@@ -34,24 +34,24 @@ gaussian_kernel = function(xy_length = 2, sigma = 1.0, range_gauss = 2) {
 #' @return a matrix
 #' @export
 #' @examples
-#' 
+#'
 #' set.seed(1)
 #' mt = matrix(1:48, 8, 6)
-#' 
+#'
 #' res = norm_matrix_range(mt, min_value = -1, max_value = 1)
-#' 
+#'
 
 norm_matrix_range = function(data, min_value = -1, max_value = 1) {
-  
+
   if (!inherits(data, 'matrix')) {
     stop('data should be a matrix')
   }
-  
+
   MIN = min(data)
   data = (data - MIN) / (max(data) - MIN)
   rng = min_value - max_value
   out = min_value - data * rng
-  
+
   return(out)
 }
 
@@ -101,7 +101,7 @@ switch_filter = function(kernel, conv_mod, gaussian_dims = 5, sigma = 1.0, lapla
 
   if (kernel == 'LoG') {                  # a discrete kernel approximation for laplacian of Gaussian for a sigma of 1.4, http://homepages.inf.ed.ac.uk/rbf/HIPR2/log.htm
 
-    gaus_kern = norm_matrix_range(gaussian_kernel(gaussian_dims, sigma, range_gauss = 2), 0.0, 255.0)        # pixels between 0 and 255  (by default)  
+    gaus_kern = norm_matrix_range(gaussian_kernel(gaussian_dims, sigma, range_gauss = 2), 0.0, 255.0)        # pixels between 0 and 255  (by default)
 
     lapl_kern = laplacian_kernels(laplacian_type)
 
@@ -161,10 +161,10 @@ func_chech_range = function(image) {
     image = Normalize_array(image)
   }
   else {
-    
+
     return(image)
   }
-  
+
   return(image)
 }
 
@@ -187,7 +187,7 @@ func_chech_range = function(image) {
 #' 'Prewitt', 'Roberts_cross', 'Scharr', 'Sobel'
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -197,68 +197,68 @@ func_chech_range = function(image) {
 
 
 edge_detection = function(image, method = NULL, conv_mode = 'same', approx = F, gaussian_dims = 5, sigma = 1.0, range_gauss = 2, laplacian_type = 1) {
-  
+
   if (is.null(method) || !method %in% c('Frei_chen', 'LoG', 'Prewitt', 'Roberts_cross', 'Scharr', 'Sobel')) stop("method shoud be non-NULL and one of : 'Frei_chen', 'LoG', 'Prewitt', 'Roberts_cross', 'Scharr', 'Sobel'")
   if (inherits(image, 'data.frame')) image = as.matrix(image)                    # default conversion for armadillo function conv2d
   if (is.null(conv_mode)) stop("conv_mode should be one of : 'full', 'same'")
   if (!conv_mode %in% c('full', 'same')) stop("conv_mode should be one of : 'full', 'same'")
-  
+
   if (!approx %in% c(T, F)) stop("the 'approx' argument should be a boolean")
-  
+
   res_kernel = switch_filter(method, conv_mode, gaussian_dims = gaussian_dims, sigma = sigma, laplacian_type = laplacian_type, range_gauss = range_gauss)
-  
+
   if (inherits(res_kernel, 'list')) {
-    
+
     if (inherits(image, 'array') && !is.na(dim(image)[3]) && dim(image)[3] == 3) {
-      
+
       new_image_horizontal = conv3d(image, res_kernel$G_horiz, conv_mode)
       new_image_vertical = conv3d(image, res_kernel$G_vert, conv_mode)
     }
     else if (inherits(image, 'matrix')) {
-      
+
       new_image_horizontal = conv2d(image, res_kernel$G_horiz, conv_mode)
       new_image_vertical = conv2d(image, res_kernel$G_vert, conv_mode)
     }
     else {
-      
+
       stop("the 'image' parameter can be either a matrix or a 3-dimensional array")
     }
-    
+
     if (approx) {
-      
+
       res = abs(new_image_horizontal) + abs(new_image_vertical)}                   # an approximate calculation is much faster, http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.301.927&rep=rep1&type=pdf
-    
+
     else {
-      
+
       res = sqrt((new_image_horizontal) ^ 2 + (new_image_vertical) ^ 2)
     }
-    
+
     res = func_chech_range(res)
-    
+
     return(res)
   }
-  
+
   else if (inherits(res_kernel, 'matrix')) {
-    
+
     if (is.array(image) && !is.na(dim(image)[3]) && dim(image)[3] == 3) {
-      
+
       new_LoG = conv3d(image, res_kernel, conv_mode)
     }
     else if (is.matrix(image)) {
-      
+
       new_LoG = conv2d(image, res_kernel, mode = conv_mode)
     }
     else {
       stop("the 'new_LoG' parameter can be either a matrix or a 3-dimensional array")
     }
-    
+
     new_LoG = func_chech_range(new_LoG)
-    
+
     return(new_LoG)
   }
-  
+
   else {
-    
+
     stop('invalid kernel object')
   }
 }
@@ -276,7 +276,7 @@ edge_detection = function(image, method = NULL, conv_mode = 'same', approx = F,
 #' This function applies a uniform filter to a matrix or to a 3-dimensional array
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -285,26 +285,26 @@ edge_detection = function(image, method = NULL, conv_mode = 'same', approx = F,
 #'
 
 uniform_filter = function(image, size, conv_mode = 'same') {
-  
+
   if (!is.vector(size)) stop('The size argument must be a vector specifying the dimensions of the uniform filter such as c(3,3)')
   if (is.null(conv_mode)) stop("conv_mode should be one of : 'full', 'same'")
   if (!conv_mode %in% c('full', 'same')) stop("conv_mode should be one of : 'full', 'same'")
-  
+
   unif_filt = matrix(1, ncol = size[1], nrow = size[2])/(size[1] * size[2])
-  
+
   if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) {
-    
+
     out = conv3d(image, unif_filt, conv_mode)
   }
   else if (is.matrix(image)) {
-    
+
     out = conv2d(image, unif_filt, conv_mode)
   }
   else {
-    
+
     stop('valid type of input-images is array or matrix')
   }
-  
+
   return(out)
 }
 
@@ -319,7 +319,7 @@ uniform_filter = function(image, size, conv_mode = 'same') {
 #' This function applies thresholding to a matrix or to a 3-dimensional array.
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -329,7 +329,7 @@ uniform_filter = function(image, size, conv_mode = 'same') {
 
 
 image_thresholding = function(image, thresh) {
-  
+
   if (thresh <= 0.0) stop('the thresh parameter should be greater than 0')
   if (inherits(image, 'data.frame')) image = as.matrix(image)
 
@@ -338,7 +338,7 @@ image_thresholding = function(image, thresh) {
     image_out = ifelse(image > thresh, 1, 0)
   }
   else if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) {
-    
+
     image = rgb_2gray(image)
     image_out = ifelse(image > thresh, 1, 0)
   }
@@ -362,7 +362,7 @@ image_thresholding = function(image, thresh) {
 #' This function applies gamma correction to a matrix or to a 3-dimensional array. The gamma correction controls the overall brightness of an image.
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "2.jpg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -415,7 +415,7 @@ sec_gaus_bl = function(image, factor, sigma, range_gauss) {
 #' This function downsamples an image with the option to use gaussian blur for optimal output.
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "2.jpg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -425,7 +425,7 @@ sec_gaus_bl = function(image, factor, sigma, range_gauss) {
 
 
 down_sample_image = function(image, factor, gaussian_blur = FALSE, gauss_sigma = 1.0, range_gauss = 2) {
-  
+
   if (!is.logical(gaussian_blur)) stop("'gaussian_blur should be one of TRUE, FALSE")
   if (inherits(image, 'data.frame')) image = as.matrix(image)
   if (!inherits(image, c('matrix', 'array'))) stop('invalid type of image, use either array or matrix')
@@ -449,7 +449,7 @@ down_sample_image = function(image, factor, gaussian_blur = FALSE, gauss_sigma =
       new_array = array(unlist(new_array), dim = c(nrow(new_array[[1]]), ncol(new_array[[1]]), length(new_array)))
       out = new_array[new_rows, new_cols, ]
     }
-    
+
     if (is.matrix(image)) {
 
       new_array = sec_gaus_bl(image, factor, gauss_sigma, range_gauss)
@@ -458,9 +458,9 @@ down_sample_image = function(image, factor, gaussian_blur = FALSE, gauss_sigma =
   }
 
   else {
-    
+
     if (inherits(image, 'matrix')) {
-      
+
       out = image[new_rows, new_cols]
     }
 
@@ -469,7 +469,7 @@ down_sample_image = function(image, factor, gaussian_blur = FALSE, gauss_sigma =
       out = image[new_rows, new_cols, ]
     }
     else {
-      
+
       stop('invalid type of image, use either array or matrix')
     }
   }
@@ -486,25 +486,35 @@ down_sample_image = function(image, factor, gaussian_blur = FALSE, gauss_sigma =
 
 crop_image_secondary = function(image, new_width, new_height) {         # reduce image size for 'equal_spaced'
 
-  r = nrow(image);
-  c = ncol(image);
+  r = nrow(image)
+  c = ncol(image)
 
-  if (r <= new_width || c <= new_height) { stop("new_height or new_width should be less than or equal to the initial dimensions of the image") }
+  if (new_width > r) stop("The 'new_width' parameter should be less than or equal to the input rows of the image", call. = F)
+  if (new_height > c) stop("The 'new_height' parameter should be less than or equal to the input columns of the image", call. = F)
 
-  dif_rows = r - new_width
-  dif_cols = c - new_height
+  if (new_width != r) {
+    dif_rows = r - new_width
+    rem_rows = dif_rows %% 2
+    keep_rows = (floor(dif_rows/2) + 1):(r - floor(dif_rows/2) - rem_rows)
+  }
+  else {
+    keep_rows = 1:r
+  }
 
-  rem_rows = dif_rows %% 2
-  rem_cols = dif_cols %% 2
-
-  remov_rows = (floor(dif_rows/2) + 1):(r - floor(dif_rows/2) - rem_rows)
-  remov_cols = (floor(dif_cols/2) + 1):(c - floor(dif_cols/2) - rem_cols)
-
-  return(image[remov_rows, remov_cols])
+  if (new_height != c) {
+    dif_cols = c - new_height
+    rem_cols = dif_cols %% 2
+    keep_cols = (floor(dif_cols/2) + 1):(c - floor(dif_cols/2) - rem_cols)
+  }
+  else {
+    keep_cols = 1:c
+  }
 
+  return(image[keep_rows, keep_cols])
 }
 
 
+
 #' crop an image
 #'
 #' @param image matrix or 3-dimensional array
@@ -517,7 +527,7 @@ crop_image_secondary = function(image, new_width, new_height) {         # reduce
 #' This function crops an image in two different ways.
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "2.jpg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -594,14 +604,14 @@ cropImage = function(image, new_width, new_height, type = 'equal_spaced') {
 #' @param image matrix, data frame or 3-dimensional array
 #' @param angle specifies the number of degrees
 #' @param method a string specifying the interpolation method when rotating an image ( 'nearest', 'bilinear' )
-#' @param mode one of 'full', 'same' (same indicates that the ouput image will have the same dimensions with initial image) 
+#' @param mode one of 'full', 'same' (same indicates that the ouput image will have the same dimensions with initial image)
 #' @param threads the number of cores to run in parallel
 #' @return depending on the input, either a matrix or an array
 #' @details
 #' This function rotates an image by a user-specified angle
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "2.jpg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -611,35 +621,35 @@ cropImage = function(image, new_width, new_height, type = 'equal_spaced') {
 
 
 rotateImage = function(image, angle, method = 'nearest', mode = 'same', threads = 1) {
-  
+
   if (threads < 1) stop('threads should be greater than 0')
   if (angle > 360.0 || angle < 0.0) stop("valid angles to rotate an image are values greater than 0 and less than 360")
   if (!method %in% c('nearest', 'bilinear')) stop("valid methods are 'nearest', 'bilinear'")
   if (!mode %in% c('same', 'full')) stop("invalid mode, choose one of 'same', 'full'")
-  
+
   if (inherits(image, 'data.frame')) image = as.matrix(image)
-  
+
   if (inherits(image, 'matrix')) {
-    
+
     out = rotate_nearest_bilinear(image, angle, method, mode, threads)
   }
   else if (inherits(image, 'array') && !is.na(dim(image)[3]) && dim(image)[3] == 3) {
-    
+
     if (mode == 'same') {
-      
+
       out = rotate_nearest_bilinear_array_same(image, angle, method, threads)
     }
     if (mode == 'full') {
-      
+
       out = rotate_nearest_bilinear_array_full(image, angle, method, threads)
     }
   }
-  
+
   else {
-    
+
     stop('invalid type of image, supported types are matrix, data frame and 3 dimensional array')
   }
-  
+
   return(out)
 }
 
@@ -653,7 +663,7 @@ rotateImage = function(image, angle, method = 'nearest', mode = 'same', threads
 #' This function is faster than the rotateImage function as it rotates an image for specific angles (90, 180 or 270 degrees).
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "3.jpeg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -731,7 +741,7 @@ sec_resiz_array = function(image, flag = T) {
 #' This function down- or upsamples an image using the 'nearest neighbors' or the 'bilinear' method
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "2.jpg", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -845,11 +855,11 @@ flipImage = function(image, mode = 'horizontal') {
 #' @details
 #' Whitening (or sphering) is the preprocessing needed for some algorithms. If we are training on images, the raw input is redundant, since adjacent
 #' pixel values are highly correlated. When using whitening the features become less correlated and all features have the same variance.
-#' @references 
+#' @references
 #' http://ufldl.stanford.edu/wiki/index.php/Whitening
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
@@ -893,46 +903,46 @@ ZCAwhiten = function(image, k, epsilon) {
 #' @param threads number of cores to run in parallel ( > 1 should be used if image high dimensional )
 #' @return a matrix or 3-dimensional array
 #' @details
-#' This function utilizes a kernel to perform delation or erosion. The first value of the vector indicates the number of rows of the kernel, whereas 
+#' This function utilizes a kernel to perform delation or erosion. The first value of the vector indicates the number of rows of the kernel, whereas
 #' the second value indicates the number of columns.
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
 #'
 #' res_delate = delationErosion(image, Filter = c(3,3), method = 'delation')
-#' 
+#'
 #' res_erode = delationErosion(image, Filter = c(5,5), method = 'erosion')
 #'
 
 
 delationErosion = function(image, Filter, method = 'delation', threads = 1) {
-  
+
   if (inherits(image, 'data.frame')) image = as.matrix(image)
   if (!inherits(image, c('matrix', 'array'))) stop('invalid type of image, use either array or matrix')
   if (threads < 1) stop('theads should be at least 1')
   if (!method %in% c('delation', 'erosion')) stop("invalid method, choose one of 'delation', 'erosion'")
-  if (length(Filter) != 2 || Filter[1] < 1 || Filter[2] < 1 || Filter[1] > nrow(image) - 1 || Filter[2] > ncol(image) - 1 || (!inherits(Filter, 'numeric'))) 
-    stop('Filter should be a numeric vector, such as c(3,3), where each value of the vector is greater than 1 and less than the number of 
+  if (length(Filter) != 2 || Filter[1] < 1 || Filter[2] < 1 || Filter[1] > nrow(image) - 1 || Filter[2] > ncol(image) - 1 || (!inherits(Filter, 'numeric')))
+    stop('Filter should be a numeric vector, such as c(3,3), where each value of the vector is greater than 1 and less than the number of
          rows and columns of the image')
-  
+
   if (method == 'delation') {
     method = 1}
   if (method == 'erosion') {
     method = 2
   }
-  
+
   if (inherits(image, 'matrix')) {
 
     res = diate_erode(image, Filter, method, threads)
   }
   if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) {
-    
+
     res = diate_erode_cube(image, Filter, method, threads)
   }
-  
+
   return(res)
 }
 
@@ -948,94 +958,94 @@ delationErosion = function(image, Filter, method = 'delation', threads = 1) {
 #' @param padded_value either a numeric value or a numeric vector of length 3 (corresponding to RGB). If it's not equal to 0 then the values of the shifted rows or columns will be filled with the user-defined padded_value
 #' @return a matrix or 3-dimensional array
 #' @details
-#' If shift_rows is not zero then the image will be sifted row-wise (upsides or downsides depending on the sign). If shift_cols is not zero then 
+#' If shift_rows is not zero then the image will be sifted row-wise (upsides or downsides depending on the sign). If shift_cols is not zero then
 #' the image will be sifted column-wise (right or left depending on the sign).
 #' @export
 #' @examples
-#' 
+#'
 #' path = system.file("tmp_images", "1.png", package = "OpenImageR")
 #'
 #' image = readImage(path)
 #'
 #' res_tr = translation(image, shift_rows = 10, shift_cols = -10)
-#' 
+#'
 
 
 
 translation = function(image, shift_rows = 0, shift_cols = 0, padded_value = 0) {
-  
+
   if (inherits(image, 'data.frame')) image = as.matrix(image)
   if (shift_rows == 0 && shift_cols == 0) stop("one of shift_rows, shift_cols should be non zero")
 
   if (inherits(image, 'matrix')) {
-    
+
     if (length(padded_value) != 1) {
-     
-      stop("the padded_value parameter should be a numeric value", call. = F) 
+
+      stop("the padded_value parameter should be a numeric value", call. = F)
     }
-    
+
     out = translation_mat(image, shift_rows, shift_cols, padded_value)                 # padded_value is a numeric value
   }
-  else if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) { 
-    
+  else if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) {
+
     if (length(padded_value) == 1) {
-      
+
       padded_value = rep(padded_value, 3)                                               # here by default arrays are 3-dimensional (padded_value vector of length 3)
     }
     else {
-      
+
       if (length(padded_value) > 1 && length(padded_value) != 3) {
-        
+
         stop("the padded_value parameter should be a vector of length 3", call. = F)
       }
     }
-    
+
     out = list()
-    
+
     for (i in 1:3) {
-      
+
       out[[i]] = translation_mat(image[,,i], shift_rows, shift_cols, padded_value[i])
     }
-    
+
     out = list_2array_convert(out)
   }
-  
+
   else {
-    
+
     stop('invalid data type, use one of matrix, data frame or array')
   }
-  
+
   return(out)
 }
 
 
 
 #' convert a list of matrices to an array of matrices
-#' 
+#'
 #' @param data a list of matrices
 #' @param verbose if TRUE then the time taken to complete the task will be printed
 #' @return an array
 #' @author Lampros Mouselimis
 #' @details
-#' This is a helper function mainly for the HOG and hash functions. In case that matrices are stored in a list, 
+#' This is a helper function mainly for the HOG and hash functions. In case that matrices are stored in a list,
 #' this function converts the list to an array of 2-dimensional data.
 #' @export
 #' @examples
-#' 
+#'
 #' lst = list(matrix(0, 100, 100), matrix(1, 100, 100))
-#' 
+#'
 #' arr = List_2_Array(lst, verbose = FALSE)
 #'
 
 
 List_2_Array = function(data, verbose = FALSE) {
-  
-  if (!inherits(data, 'list')) stop('the data should be a list') 
+
+  if (!inherits(data, 'list')) stop('the data should be a list')
   if (sum(unlist(lapply(data, function(x) inherits(x, 'matrix')))) != length(data)) stop('the sublists should be of type matrix')
   if (sum(unlist(lapply(data, nrow)) == nrow(data[[1]])) != length(data) || sum(unlist(lapply(data, ncol)) == ncol(data[[1]])) != length(data)) stop('dimension mismatch of columns or rows of the sublists')
-  
+
   if (verbose) {
-    
+
     start = Sys.time()
     out = list_2array_convert(data)
     end = Sys.time()
@@ -1043,18 +1053,18 @@ List_2_Array = function(data, verbose = FALSE) {
     cat('\n'); cat('time to complete :', t, attributes(t)$units, '\n'); cat('\n');
   }
   else {
-    
+
     out = list_2array_convert(data)
   }
-  
+
   return(out)
 }
 
 
 
 #' image augmentations of a matrix, data frame, array or a list of 3-dimensional arrays
-#' 
-#' 
+#'
+#'
 #' @param image a matrix, data frame, array or list of 3-dimensional arrays
 #' @param flip_mode a character string ('horizontal', 'vertical')
 #' @param crop_width an integer specifying the new width of the image, after the image is cropped. Corresponds to the image-rows.
@@ -1066,7 +1076,7 @@ List_2_Array = function(data, verbose = FALSE) {
 #' @param shift_cols a positive or negative integer specifying the direction that the columns should be shifted
 #' @param rotate_angle an integer specifying the rotation angle of the image
 #' @param rotate_method a string specifying the interpolation method when rotating an image ('nearest', 'bilinear')
-#' @param zca_comps an integer specifying the number of components to keep by zca whitening, when svd is performed 
+#' @param zca_comps an integer specifying the number of components to keep by zca whitening, when svd is performed
 #' @param zca_epsilon a float specifying the regularization parameter by zca whitening
 #' @param image_thresh the threshold parameter, by image thresholding, should be between 0 and 1 if the data is normalized or between 0-255 otherwise
 #' @param padded_value either a numeric value or a numeric vector of length equal to N of an N-dimensional array. If it's not equal to 0 then the values of the shifted rows or columns will be filled with the user-defined padded_value. Applies only to the shift_rows and shift_cols parameters.
@@ -1078,39 +1088,39 @@ List_2_Array = function(data, verbose = FALSE) {
 #' is : 1st flip image, 2nd crop image, 3rd resize image, 4th shift rows or columns, 5th rotate image, 6th zca-whitening and 7th image-thresholding.
 #' @export
 #' @examples
-#' 
+#'
 #' \dontrun{
-#' 
+#'
 #' # a matrix
 #' object = matrix(1, 10, 10)
-#' 
+#'
 #' res = Augmentation(object, resiz_width = 8, resiz_height = 8, rotate_angle = 40)
-#' 
-#' 
+#'
+#'
 #' # an array
 #' object = array(0, dim = c(10, 10, 3))
-#' 
+#'
 #' res = Augmentation(object, resiz_width = 8, resiz_height = 8, rotate_angle = 30)
-#' 
-#' 
+#'
+#'
 #' # an array (multiple matrices)
 #' object = array(0, dim = c(10, 10, 10))
-#' 
+#'
 #' res = Augmentation(object, resiz_width = 8, resiz_height = 8, rotate_angle = 20)
-#' 
-#' 
+#'
+#'
 #' # a list of 3-dimensional arrays
-#' object = list(array(0, dim = c(10, 10, 3)), array(0, dim = c(10, 10, 3)))                
-#' 
+#' object = list(array(0, dim = c(10, 10, 3)), array(0, dim = c(10, 10, 3)))
+#'
 #' res = Augmentation(object, resiz_width = 8, resiz_height = 8, rotate_angle = 40)
 #' }
 
 
 Augmentation = function(image, flip_mode = NULL, crop_width = NULL, crop_height = NULL, resiz_width = 0, resiz_height = 0, resiz_method = "nearest", shift_rows = 0,
-                        
+
                         shift_cols = 0, rotate_angle = 0, rotate_method = "nearest", zca_comps = 0, zca_epsilon = 0.0, image_thresh = 0.0, padded_value = 0, verbose = FALSE) {
-  
-  
+
+
   if (!inherits(image, c('data.frame', 'matrix', 'array', 'list'))) stop('the image parameter should be either a matrix, data frame, array or a list')
   if ((length(crop_height) < 2 && !is.null(crop_height)) || (length(crop_width) < 2 && !is.null(crop_width))) stop('crop_height and crop_width should be a sequence of numeric values')
   if (length(resiz_width) != 1 || length(resiz_height) != 1) stop('resiz_height and resiz_width should be a single value')
@@ -1133,68 +1143,68 @@ Augmentation = function(image, flip_mode = NULL, crop_width = NULL, crop_height
   if (is.null(crop_width)) crop_width = numeric(0)
   if (!is.null(flip_mode) && !flip_mode %in% c("horizontal", "vertical")) stop("valid flip_mode is one of 'horizontal', 'vertical'")
   if (is.null(flip_mode)) flip_mode = ""
-  
+
   if (verbose) start = Sys.time()
-  
+
   if (inherits(image, 'matrix')) {
-    
+
     if (length(padded_value) != 1) {
-      
-      stop("the padded_value parameter should be a numeric value", call. = F) 
+
+      stop("the padded_value parameter should be a numeric value", call. = F)
     }
-    
-    out = augment_transf(image, flip_mode, crop_height, crop_width, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle, rotate_method, 
-                         
+
+    out = augment_transf(image, flip_mode, crop_height, crop_width, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle, rotate_method,
+
                          zca_comps, zca_epsilon, image_thresh, padded_value)}
-  
+
   if (inherits(image, 'array') && !is.na(dim(image)[3])) {             # here arrays can be >= 3-dimensional ( besides RGB-images also an array of matrices )
-    
+
     if (length(padded_value) == 1) {
-      
+
       padded_value = rep(padded_value, dim(image)[3])
     }
     else {
-      
+
       if (length(padded_value) > 1 && length(padded_value) != dim(image)[3]) {
-        
+
         stop(paste0("the padded_value parameter should be a vector of length equal to ", dim(image)[3]), call. = F)
       }
     }
-    
-    out = augment_transf_array(image, flip_mode, crop_height, crop_width, padded_value, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle, 
-                               
+
+    out = augment_transf_array(image, flip_mode, crop_height, crop_width, padded_value, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle,
+
                                rotate_method, zca_comps, zca_epsilon, image_thresh)}
-  
+
   if (inherits(image, 'list')) {
-    
+
     if ( sum(unlist(lapply(image, function(x) inherits(x, 'array') && dim(x)[3] == 3))) != length(image) ) stop('the list should consist of 3-dimensional arrays')
     if ( sum(unlist(lapply(image, function(x) nrow(x) <= length(crop_width)))) ) stop('the length of the crop_width sequence should be less than or equal to the initial width of each of the images')
     if ( sum(unlist(lapply(image, function(x) ncol(x) <= length(crop_height)))) ) stop('the length of the crop_height sequence should be less than or equal to the initial height of each of the images')
-    
+
     if (length(padded_value) == 1) {
-      
+
       padded_value = rep(padded_value, 3)
     }
     else {
-      
+
       if (length(padded_value) > 1 && length(padded_value) != 3) {
-        
+
         stop("the padded_value parameter should be a vector of length 3", call. = F)
       }
     }
-    
-    out = augment_array_list(image, flip_mode, crop_height, crop_width, padded_value, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle, rotate_method, 
-                             
+
+    out = augment_array_list(image, flip_mode, crop_height, crop_width, padded_value, resiz_width, resiz_height, resiz_method, shift_rows, shift_cols, rotate_angle, rotate_method,
+
                              zca_comps, zca_epsilon, image_thresh)
   }
-  
+
   if (verbose) {
-    
+
     end = Sys.time()
     t = end - start
     cat('\n'); cat('time to complete :', t, attributes(t)$units, '\n'); cat('\n');
   }
-  
+
   return(out)
 }
 
@@ -1206,56 +1216,56 @@ Augmentation = function(image, flip_mode = NULL, crop_width = NULL, crop_height
 #' @return either a normalized vector, matrix, or array
 #' @author Lampros Mouselimis
 #' @details
-#' This is a helper function which normalizes all pixel values of the object to the range between 0 and 1. The function takes either a vector, matrix, data frame or 
+#' This is a helper function which normalizes all pixel values of the object to the range between 0 and 1. The function takes either a vector, matrix, data frame or
 #' array as input and returns a normalized object of the same type (in case of data frame it returns a matrix).
 #' @export
 #' @examples
-#' 
+#'
 #' # vector
 #' x = 1:10
-#' 
+#'
 #' res = NormalizeObject(x)
-#' 
-#' 
+#'
+#'
 #' # matrix
 #' x = matrix(runif(100), 10, 10)
-#' 
+#'
 #' res = NormalizeObject(x)
-#' 
-#' 
+#'
+#'
 #' # data frame
 #' x = data.frame(matrix(runif(100), 10, 10))
-#' 
+#'
 #' res = NormalizeObject(x)
-#' 
-#' 
+#'
+#'
 #' # array
-#' x = array(runif(100), dim = c(10, 10, 3))               
-#' 
+#' x = array(runif(100), dim = c(10, 10, 3))
+#'
 #' res = NormalizeObject(x)
 #'
 
 NormalizeObject = function(x) {
-  
+
   if (inherits(x, 'data.frame')) x = as.matrix(x)
-  
+
   if (is.vector(x, mode = 'numeric')) {
-    
+
     out = (x - min(x))/(max(x) - min(x))
   }
   else if (inherits(x, 'matrix')) {
-    
+
     out = Normalize_matrix(x)
   }
-  else if (all(c(inherits(x, 'array'), !is.na(dim(x)[3]), dim(x)[3] == 3))) { 
-    
+  else if (all(c(inherits(x, 'array'), !is.na(dim(x)[3]), dim(x)[3] == 3))) {
+
     out = Normalize_array(x)
   }
   else {
-    
+
     stop('invalid data type, use one of vector, matrix, data frame or array')
   }
-  
+
   return(out)
 }
 
@@ -1267,57 +1277,57 @@ NormalizeObject = function(x) {
 #' @return a list
 #' @author Lampros Mouselimis
 #' @details
-#' This helper function returns the minimum and maximum values of a vector, 2-dimensional or 3-dimensional objects. In case of a vector, matrix or data frame it returns a single value for 
+#' This helper function returns the minimum and maximum values of a vector, 2-dimensional or 3-dimensional objects. In case of a vector, matrix or data frame it returns a single value for
 #' the minimum and maximum of the object. In case of an array it returns the minimum and maximum values for each slice of the array.
 #' @export
 #' @examples
-#' 
+#'
 #' # vector
 #' x = 1:10
-#' 
+#'
 #' res = MinMaxObject(x)
-#' 
-#' 
+#'
+#'
 #' # matrix
 #' x = matrix(runif(100), 10, 10)
-#' 
+#'
 #' res = MinMaxObject(x)
-#' 
-#' 
+#'
+#'
 #' # data frame
 #' x = data.frame(matrix(runif(100), 10, 10))
-#' 
+#'
 #' res = MinMaxObject(x)
-#' 
-#' 
+#'
+#'
 #' # array
-#' x = array(runif(100), dim = c(10, 10, 3))               
-#' 
+#' x = array(runif(100), dim = c(10, 10, 3))
+#'
 #' res = MinMaxObject(x)
 #'
 
 
 MinMaxObject = function(x) {
-  
+
   if (inherits(x, 'data.frame')) x = as.matrix(x)
-  
+
   if (is.vector(x, mode = 'numeric')) {
-    
+
     out = list(min = min(x), max = max(x))
   }
   else if (inherits(x, 'matrix')) {
-    
+
     out = MinMaxMatrix(x)
   }
-  else if (all(c(inherits(x, 'array'), !is.na(dim(x)[3]), dim(x)[3] == 3))) { 
-    
+  else if (all(c(inherits(x, 'array'), !is.na(dim(x)[3]), dim(x)[3] == 3))) {
+
     out = MinMaxArray(x)
   }
   else {
-    
+
     stop('invalid data type, use one of vector, matrix, data frame or array')
   }
-  
+
   return(out)
 }
 
@@ -1334,45 +1344,45 @@ MinMaxObject = function(x) {
 #' when mode 'full' the rows and columns of the output object equals : ROWS = nrow(image) + nrow(kernel) - 1 and COLUMNS = ncol(image) + ncol(kernel) - 1
 #' @export
 #' @examples
-#' 
-#' 
+#'
+#'
 #' # kernel
 #' x = matrix(1, nrow = 4, ncol = 4) / 16   # uniform
-#' 
-#' 
+#'
+#'
 #' # matrix
 #' image_matrix = matrix(runif(100), 10, 10)
-#' 
+#'
 #' res = convolution(image_matrix, x, "same")
-#' 
-#' 
+#'
+#'
 #' # array
 #' image_array = array(runif(100), dim = c(10, 10, 3))
-#' 
+#'
 #' res = convolution(image_array, x, "same")
-#' 
+#'
 
 
 
 convolution = function(image, kernel, mode = "same") {
-  
+
   if (!inherits(kernel, 'matrix')) stop('the kernel should be a matrix')
   if (!mode %in% c('same', 'full')) stop('the mode should one of same, full')
   if (inherits(image, 'data.frame')) image = as.matrix(image)
-  
+
   if (inherits(image, 'matrix')) {
-    
+
     out = conv2d(image, kernel, mode)
   }
   else if (all(c(inherits(image, 'array'), !is.na(dim(image)[3]), dim(image)[3] == 3))) {
-    
+
     out = conv3d(image, kernel, mode)
   }
   else {
-    
+
     stop('invalid type of image, choose one of data frame, matrix or 3 dimensional array')
   }
-  
+
   return(out)
 }
 
@@ -1393,10 +1403,10 @@ convolution = function(image, kernel, mode = "same") {
 #' image = readImage(path)
 #'
 #' gray = rgb_2gray(image)
-#' 
+#'
 
 rgb_2gray = function(RGB_image) {
   if (!all(c(inherits(RGB_image, 'array'), !is.na(dim(RGB_image)[3]), dim(RGB_image)[3] == 3))) stop("The 'RGB_image' parameter must be a 3-dimensional array!")
-  
+
   return(Rgb_2gray(RGB_image))
 }
