@@ -352,6 +352,8 @@ namespace oimageR {
 
 
       // similar to the 'x' matrix of the matlab 'meshgrid' function  [ populating the matrices column-wise, is faster ]
+      // Armadillo uses the "column-major ordering" which means read and write by column is faster than by row, see:
+      // https://stackoverflow.com/a/53498100
       //
 
       arma::mat meshgrid_x(int rows, int cols) {
@@ -382,6 +384,8 @@ namespace oimageR {
 
 
       // similar to the 'y' matrix of the matlab 'meshgrid' function  [ populating the matrices column-wise, is faster ]
+      // Armadillo uses the "column-major ordering" which means read and write by column is faster than by row, see:
+      // https://stackoverflow.com/a/53498100
       //
 
       arma::mat meshgrid_y(int rows, int cols) {
@@ -550,8 +554,75 @@ namespace oimageR {
         }
       }
 
+      
+      
+      // padding of new rows / cols based on the 'new_rows' and 'new_cols' parameters
+      //
+      
+      Rcpp::List pad_matrix(arma::mat &x, int new_rows, int new_cols, double fill_value = 0.0) {
+        
+        if (new_rows < x.n_rows) {
+          Rcpp::stop("The 'new_rows' should be greater than the rows of the input data");
+        }
+        if (new_cols < x.n_cols) {
+          Rcpp::stop("The 'new_cols' should be greater than columns of the input data");
+        }
+        
+        unsigned int dif_rows = new_rows - x.n_rows;
+        arma::mat concat_row(1, x.n_cols);
+        
+        int rows_start = 0;
+        int rows_end = 0;
+        
+        if (dif_rows > 0) {
+          
+          concat_row.fill(fill_value);
+          
+          for (unsigned int i = 0; i < dif_rows; i++) {
+            
+            if (EVEN(i)) {
+              x = arma::join_cols(x, concat_row);
+              rows_end++;
+            }
+            else {
+              x = arma::join_cols(concat_row, x);
+              rows_start++;
+            }
+          }
+        }
+        
+        arma::mat concat_col(x.n_rows, 1);
+        unsigned int dif_cols = new_cols - x.n_cols;
+        
+        int cols_start = 0;
+        int cols_end = 0;
+        
+        if (dif_cols > 0) {
+          
+          concat_col.fill(fill_value);
+          
+          for (unsigned int i = 0; i < dif_cols; i++) {
+            
+            if (EVEN(i)) {
+              x = arma::join_rows(x, concat_col);
+              cols_end++;
+            }
+            else {
+              x = arma::join_rows(concat_col, x);
+              cols_start++;
+            }
+          }
+        }
+        
+        return Rcpp::List::create( Rcpp::Named("data") = x,
+                                   Rcpp::Named("padded_start") = rows_start,
+                                   Rcpp::Named("padded_end") = rows_end,
+                                   Rcpp::Named("padded_left") = cols_start,
+                                   Rcpp::Named("padded_right") = cols_end );
+      }
 
 
+      
       // replace value above or below a thresh
       // if mode = 1 : thresh >   (if greater)
       // if mode = 2 : thresh <   (if less)
